﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Net;
using System.IO;

using System.Web; //Add reference to System.Web
using System.Runtime.Serialization.Json; //Add reference to System.Runtime.Serialization

namespace RedCritterConnecter.Samples
{
    //Create a class to contain the response
    public class AdjustSkillResponse
    {
        public String ErrorMessage { get; set; }
        public Boolean Submitted { get; set; }
        public String RequestID { get; set; }
        public String APIVersion { get; set; }
        public String ReturnParam1 { get; set; }
        public String ReturnParam2 { get; set; }
        public String ReturnParam3 { get; set; }
        private Int32 ErrorCode { get; set; }
    }

    public class AdjustSkill
    {

        public delegate void OnAdjustSkillResponse(AdjustSkillResponse AdjustSkillResponse);
        public event OnAdjustSkillResponse onAdjustSkillResponse;

        public delegate void OnAdjustSkillResponseError(AdjustSkillResponse AdjustSkillResponse);
        public event OnAdjustSkillResponseError onAdjustSkillResponseError;

        const string CONST_ConnecterBaseURL = "https://www.redcritterconnecter.com/";
          
        public void Execute(String SecretKey, String ExternalAccountID, String ProfileEntity, String SkillName, Double Points)
        {
            try
            {
                //Create url encoded parameters in query string
                String queryString = "secretkey=" + System.Web.HttpUtility.UrlEncode(SecretKey) + "&externalaccountid=" + System.Web.HttpUtility.UrlEncode(ExternalAccountID) + "&skillname=" + System.Web.HttpUtility.UrlEncode(SkillName) + "&profileentity=" + System.Web.HttpUtility.UrlEncode(ProfileEntity) + "&points=" + Points;

                //Create a new instance of a WebClient
                WebClient wc = new System.Net.WebClient();

                //Prevent this request from caching in order to ensure that it is sent to server
                wc.CachePolicy = new System.Net.Cache.RequestCachePolicy(System.Net.Cache.RequestCacheLevel.NoCacheNoStore);

                //Attach an event handler to receive the response
                wc.DownloadStringCompleted += onAdjustSkillResponseReceived;

                //Make the call
                wc.DownloadStringAsync(new Uri(CONST_ConnecterBaseURL + "services/gamificationv1/AdjustSkill?" + queryString, UriKind.Absolute));
            }
            catch
            {
                //Something went wrong communicating with the server
                if (onAdjustSkillResponseError != null)
                {
                    onAdjustSkillResponseError(null);
                }
            }

        }

        private void onAdjustSkillResponseReceived(object sender, DownloadStringCompletedEventArgs e)
        {
            try
            {
                //Create a JSON serializer
                System.Runtime.Serialization.Json.DataContractJsonSerializer s = new DataContractJsonSerializer(typeof(AdjustSkillResponse));

                //Copy the string into a memory stream
                MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(e.Result));

                //Read the stream into an object matching the response's type
                AdjustSkillResponse AdjustSkillResponse = (AdjustSkillResponse)s.ReadObject(ms);

                //Work with the populated response object
                if (AdjustSkillResponse.Submitted == true)
                {
                    //Request was accepted, raise the success event
                    onAdjustSkillResponse(AdjustSkillResponse);
                }
                else
                {
                    //Something went wrong
                    if (onAdjustSkillResponseError != null)
                    {
                        //Request failed, raise the error event
                        onAdjustSkillResponseError(AdjustSkillResponse);
                    }
                }
            }
            catch
            {
                //Something went wrong
                onAdjustSkillResponseError(null);
            }
        }
    }
}
